# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: descriptor_service.proto
# pylint: skip-file
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='descriptor_service.proto',
  package='chip.rpc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x18\x64\x65scriptor_service.proto\x12\x08\x63hip.rpc\"!\n\nDeviceType\x12\x13\n\x0b\x64\x65vice_type\x18\x01 \x01(\r\"\x1d\n\x07\x43luster\x12\x12\n\ncluster_id\x18\x01 \x01(\r\"\x1c\n\x08\x45ndpoint\x12\x10\n\x08\x65ndpoint\x18\x01 \x01(\r2\xf7\x01\n\nDescriptor\x12>\n\x0e\x44\x65viceTypeList\x12\x12.chip.rpc.Endpoint\x1a\x14.chip.rpc.DeviceType\"\x00\x30\x01\x12\x37\n\nServerList\x12\x12.chip.rpc.Endpoint\x1a\x11.chip.rpc.Cluster\"\x00\x30\x01\x12\x37\n\nClientList\x12\x12.chip.rpc.Endpoint\x1a\x11.chip.rpc.Cluster\"\x00\x30\x01\x12\x37\n\tPartsList\x12\x12.chip.rpc.Endpoint\x1a\x12.chip.rpc.Endpoint\"\x00\x30\x01\x62\x06proto3'
)




_DEVICETYPE = _descriptor.Descriptor(
  name='DeviceType',
  full_name='chip.rpc.DeviceType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_type', full_name='chip.rpc.DeviceType.device_type', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=71,
)


_CLUSTER = _descriptor.Descriptor(
  name='Cluster',
  full_name='chip.rpc.Cluster',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='chip.rpc.Cluster.cluster_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=73,
  serialized_end=102,
)


_ENDPOINT = _descriptor.Descriptor(
  name='Endpoint',
  full_name='chip.rpc.Endpoint',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='endpoint', full_name='chip.rpc.Endpoint.endpoint', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=104,
  serialized_end=132,
)

DESCRIPTOR.message_types_by_name['DeviceType'] = _DEVICETYPE
DESCRIPTOR.message_types_by_name['Cluster'] = _CLUSTER
DESCRIPTOR.message_types_by_name['Endpoint'] = _ENDPOINT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DeviceType = _reflection.GeneratedProtocolMessageType('DeviceType', (_message.Message,), {
  'DESCRIPTOR' : _DEVICETYPE,
  '__module__' : 'descriptor_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.DeviceType)
  })
_sym_db.RegisterMessage(DeviceType)

Cluster = _reflection.GeneratedProtocolMessageType('Cluster', (_message.Message,), {
  'DESCRIPTOR' : _CLUSTER,
  '__module__' : 'descriptor_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.Cluster)
  })
_sym_db.RegisterMessage(Cluster)

Endpoint = _reflection.GeneratedProtocolMessageType('Endpoint', (_message.Message,), {
  'DESCRIPTOR' : _ENDPOINT,
  '__module__' : 'descriptor_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.Endpoint)
  })
_sym_db.RegisterMessage(Endpoint)



_DESCRIPTOR = _descriptor.ServiceDescriptor(
  name='Descriptor',
  full_name='chip.rpc.Descriptor',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=135,
  serialized_end=382,
  methods=[
  _descriptor.MethodDescriptor(
    name='DeviceTypeList',
    full_name='chip.rpc.Descriptor.DeviceTypeList',
    index=0,
    containing_service=None,
    input_type=_ENDPOINT,
    output_type=_DEVICETYPE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ServerList',
    full_name='chip.rpc.Descriptor.ServerList',
    index=1,
    containing_service=None,
    input_type=_ENDPOINT,
    output_type=_CLUSTER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClientList',
    full_name='chip.rpc.Descriptor.ClientList',
    index=2,
    containing_service=None,
    input_type=_ENDPOINT,
    output_type=_CLUSTER,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='PartsList',
    full_name='chip.rpc.Descriptor.PartsList',
    index=3,
    containing_service=None,
    input_type=_ENDPOINT,
    output_type=_ENDPOINT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DESCRIPTOR)

DESCRIPTOR.services_by_name['Descriptor'] = _DESCRIPTOR

# @@protoc_insertion_point(module_scope)
