# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: device_service.proto
# pylint: skip-file
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gazoo_device.protos import common_pb2 as common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='device_service.proto',
  package='chip.rpc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14\x64\x65vice_service.proto\x12\x08\x63hip.rpc\x1a\x0c\x63ommon.proto\"X\n\x0bPairingInfo\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\x15\n\rdiscriminator\x18\x02 \x01(\r\x12\x0f\n\x07qr_code\x18\x03 \x01(\t\x12\x13\n\x0bqr_code_url\x18\x04 \x01(\t\"\x91\x01\n\nDeviceInfo\x12\x11\n\tvendor_id\x18\x01 \x01(\r\x12\x12\n\nproduct_id\x18\x02 \x01(\r\x12\x18\n\x10software_version\x18\x03 \x01(\r\x12\x15\n\rserial_number\x18\x04 \x01(\t\x12+\n\x0cpairing_info\x18\x05 \x01(\x0b\x32\x15.chip.rpc.PairingInfo\"0\n\nFabricInfo\x12\x11\n\tfabric_id\x18\x01 \x01(\r\x12\x0f\n\x07node_id\x18\x02 \x01(\x04\"X\n\x0b\x44\x65viceState\x12\x1e\n\x16time_since_boot_millis\x18\x01 \x01(\x04\x12)\n\x0b\x66\x61\x62ric_info\x18\x02 \x03(\x0b\x32\x14.chip.rpc.FabricInfo\"\'\n\x0cPairingState\x12\x17\n\x0fpairing_enabled\x18\x01 \x01(\x08\x32\xeb\x03\n\x06\x44\x65vice\x12\x38\n\x0c\x46\x61\x63toryReset\x12\x12.pw.protobuf.Empty\x1a\x12.pw.protobuf.Empty\"\x00\x12\x32\n\x06Reboot\x12\x12.pw.protobuf.Empty\x1a\x12.pw.protobuf.Empty\"\x00\x12\x36\n\nTriggerOta\x12\x12.pw.protobuf.Empty\x1a\x12.pw.protobuf.Empty\"\x00\x12;\n\rGetDeviceInfo\x12\x12.pw.protobuf.Empty\x1a\x14.chip.rpc.DeviceInfo\"\x00\x12=\n\x0eGetDeviceState\x12\x12.pw.protobuf.Empty\x1a\x15.chip.rpc.DeviceState\"\x00\x12?\n\x0fSetPairingState\x12\x16.chip.rpc.PairingState\x1a\x12.pw.protobuf.Empty\"\x00\x12?\n\x0fGetPairingState\x12\x12.pw.protobuf.Empty\x1a\x16.chip.rpc.PairingState\"\x00\x12=\n\x0eSetPairingInfo\x12\x15.chip.rpc.PairingInfo\x1a\x12.pw.protobuf.Empty\"\x00\x62\x06proto3'
  ,
  dependencies=[common__pb2.DESCRIPTOR,])




_PAIRINGINFO = _descriptor.Descriptor(
  name='PairingInfo',
  full_name='chip.rpc.PairingInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='chip.rpc.PairingInfo.code', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='discriminator', full_name='chip.rpc.PairingInfo.discriminator', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='qr_code', full_name='chip.rpc.PairingInfo.qr_code', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='qr_code_url', full_name='chip.rpc.PairingInfo.qr_code_url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=48,
  serialized_end=136,
)


_DEVICEINFO = _descriptor.Descriptor(
  name='DeviceInfo',
  full_name='chip.rpc.DeviceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='vendor_id', full_name='chip.rpc.DeviceInfo.vendor_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='product_id', full_name='chip.rpc.DeviceInfo.product_id', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='software_version', full_name='chip.rpc.DeviceInfo.software_version', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serial_number', full_name='chip.rpc.DeviceInfo.serial_number', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pairing_info', full_name='chip.rpc.DeviceInfo.pairing_info', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=139,
  serialized_end=284,
)


_FABRICINFO = _descriptor.Descriptor(
  name='FabricInfo',
  full_name='chip.rpc.FabricInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fabric_id', full_name='chip.rpc.FabricInfo.fabric_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='node_id', full_name='chip.rpc.FabricInfo.node_id', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=286,
  serialized_end=334,
)


_DEVICESTATE = _descriptor.Descriptor(
  name='DeviceState',
  full_name='chip.rpc.DeviceState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time_since_boot_millis', full_name='chip.rpc.DeviceState.time_since_boot_millis', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fabric_info', full_name='chip.rpc.DeviceState.fabric_info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=336,
  serialized_end=424,
)


_PAIRINGSTATE = _descriptor.Descriptor(
  name='PairingState',
  full_name='chip.rpc.PairingState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pairing_enabled', full_name='chip.rpc.PairingState.pairing_enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=426,
  serialized_end=465,
)

_DEVICEINFO.fields_by_name['pairing_info'].message_type = _PAIRINGINFO
_DEVICESTATE.fields_by_name['fabric_info'].message_type = _FABRICINFO
DESCRIPTOR.message_types_by_name['PairingInfo'] = _PAIRINGINFO
DESCRIPTOR.message_types_by_name['DeviceInfo'] = _DEVICEINFO
DESCRIPTOR.message_types_by_name['FabricInfo'] = _FABRICINFO
DESCRIPTOR.message_types_by_name['DeviceState'] = _DEVICESTATE
DESCRIPTOR.message_types_by_name['PairingState'] = _PAIRINGSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PairingInfo = _reflection.GeneratedProtocolMessageType('PairingInfo', (_message.Message,), {
  'DESCRIPTOR' : _PAIRINGINFO,
  '__module__' : 'device_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.PairingInfo)
  })
_sym_db.RegisterMessage(PairingInfo)

DeviceInfo = _reflection.GeneratedProtocolMessageType('DeviceInfo', (_message.Message,), {
  'DESCRIPTOR' : _DEVICEINFO,
  '__module__' : 'device_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.DeviceInfo)
  })
_sym_db.RegisterMessage(DeviceInfo)

FabricInfo = _reflection.GeneratedProtocolMessageType('FabricInfo', (_message.Message,), {
  'DESCRIPTOR' : _FABRICINFO,
  '__module__' : 'device_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.FabricInfo)
  })
_sym_db.RegisterMessage(FabricInfo)

DeviceState = _reflection.GeneratedProtocolMessageType('DeviceState', (_message.Message,), {
  'DESCRIPTOR' : _DEVICESTATE,
  '__module__' : 'device_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.DeviceState)
  })
_sym_db.RegisterMessage(DeviceState)

PairingState = _reflection.GeneratedProtocolMessageType('PairingState', (_message.Message,), {
  'DESCRIPTOR' : _PAIRINGSTATE,
  '__module__' : 'device_service_pb2'
  # @@protoc_insertion_point(class_scope:chip.rpc.PairingState)
  })
_sym_db.RegisterMessage(PairingState)



_DEVICE = _descriptor.ServiceDescriptor(
  name='Device',
  full_name='chip.rpc.Device',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=468,
  serialized_end=959,
  methods=[
  _descriptor.MethodDescriptor(
    name='FactoryReset',
    full_name='chip.rpc.Device.FactoryReset',
    index=0,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='Reboot',
    full_name='chip.rpc.Device.Reboot',
    index=1,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='TriggerOta',
    full_name='chip.rpc.Device.TriggerOta',
    index=2,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDeviceInfo',
    full_name='chip.rpc.Device.GetDeviceInfo',
    index=3,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_DEVICEINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDeviceState',
    full_name='chip.rpc.Device.GetDeviceState',
    index=4,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_DEVICESTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetPairingState',
    full_name='chip.rpc.Device.SetPairingState',
    index=5,
    containing_service=None,
    input_type=_PAIRINGSTATE,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetPairingState',
    full_name='chip.rpc.Device.GetPairingState',
    index=6,
    containing_service=None,
    input_type=common__pb2._EMPTY,
    output_type=_PAIRINGSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetPairingInfo',
    full_name='chip.rpc.Device.SetPairingInfo',
    index=7,
    containing_service=None,
    input_type=_PAIRINGINFO,
    output_type=common__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DEVICE)

DESCRIPTOR.services_by_name['Device'] = _DEVICE

# @@protoc_insertion_point(module_scope)
